package com.sunxung.factoring.entity.financing.review;

import com.baomidou.mybatisplus.annotation.TableField;
import com.sunxung.factoring.entity.BaseEntity;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @Description : 额度支用信息
 * @Author : L.Jia
 * @Date: 2023-12-18 21:02
 */
public class ReviewQuotaDisburseHis extends BaseEntity {

    /**
     * 额度支用Id
     */
    private Long reviewQuotaDisburseId;

    /**
     * 业务编号
     */
    private String businessNumber;

    /**
     * 支用批次
     */
    private Integer quotaDisburseBatch;

    /**
     * 业务综合授信余额
     */
    private BigDecimal businessCreditBalance;

    /**
     * 供应商授信余额
     */
    private BigDecimal supplierCreditBalance;

    /**
     * 应收账款起始日
     */
    private Date startDate;

    /**
     * 应收账款期限
     */
    private Integer term;

    /**
     * 应收账款到期日
     */
    private Date endDate;

    /**
     * 应收账款净额总计
     **/
    @TableField("accounts_receivable_net_amount_total")
    private BigDecimal accountsReceivableNetAmountTotal;

    /**
     * 账户上报方式 1:A类，0:B类
     */
    @TableField("report_method")
    private Integer reportMethod;

    /**
     * 有无相关还款责任人:0无、1有
     **/
    @TableField("is_resp_repay")
    private boolean respRepay;

    /**
     * 有无抵质押:0无、1有
     **/
    @TableField("is_mortgage_info")
    private boolean mortgageInfo;

    /**
     * 预期Irr(字符串)
     */
    @TableField(exist = false)
    private String planIrrStr;

    /**
     * 预期Irr
     */
    @TableField("plan_irr")
    private BigDecimal planIrr;

    /**
     * 融资信息
     */
    @TableField(exist = false)
    private ReviewFinancingInfo reviewFinancingInfo;

    /**
     * 主合同信息
     */
    @TableField(exist = false)
    private ReviewMasterContract reviewMasterContract;

    /**
     * 收支计划信息
     **/
    @TableField(exist = false)
    private List<ReviewIncomeExpenditurePlan> incomeExpenditurePlanList;

    /**
     * 相关还款责任人信息
     **/
    @TableField(exist = false)
    private List<ReviewRltRepay> rltRepayList;

    /**
     * 抵质押基础信息
     **/
    @TableField(exist = false)
    private List<ReviewMortgageBasicInfo> mortgageBasicInfoList;

    /**
     * 应收账款转让信息
     **/
    @TableField(exist = false)
    private ReviewTransferReceivable reviewTransferReceivable;

    /**
     * 基础交易合同信息
     */
    @TableField(exist = false)
    private List<ReviewBasicContract> reviewBasicContractList;

    public BigDecimal getBusinessCreditBalance() {
        return businessCreditBalance;
    }

    public void setBusinessCreditBalance(BigDecimal businessCreditBalance) {
        this.businessCreditBalance = businessCreditBalance;
    }

    public BigDecimal getSupplierCreditBalance() {
        return supplierCreditBalance;
    }

    public void setSupplierCreditBalance(BigDecimal supplierCreditBalance) {
        this.supplierCreditBalance = supplierCreditBalance;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Integer getTerm() {
        return term;
    }

    public void setTerm(Integer term) {
        this.term = term;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public Integer getReportMethod() {
        return reportMethod;
    }

    public void setReportMethod(Integer reportMethod) {
        this.reportMethod = reportMethod;
    }

    public ReviewMasterContract getReviewMasterContract() {
        return reviewMasterContract;
    }

    public void setReviewMasterContract(ReviewMasterContract reviewMasterContract) {
        this.reviewMasterContract = reviewMasterContract;
    }

    public ReviewFinancingInfo getReviewFinancingInfo() {
        return reviewFinancingInfo;
    }

    public void setReviewFinancingInfo(ReviewFinancingInfo reviewFinancingInfo) {
        this.reviewFinancingInfo = reviewFinancingInfo;
    }

    public Long getReviewQuotaDisburseId() {
        return reviewQuotaDisburseId;
    }

    public void setReviewQuotaDisburseId(Long reviewQuotaDisburseId) {
        this.reviewQuotaDisburseId = reviewQuotaDisburseId;
    }

    public Integer getQuotaDisburseBatch() {
        return quotaDisburseBatch;
    }

    public void setQuotaDisburseBatch(Integer quotaDisburseBatch) {
        this.quotaDisburseBatch = quotaDisburseBatch;
    }

    public List<ReviewIncomeExpenditurePlan> getIncomeExpenditurePlanList() {
        return incomeExpenditurePlanList;
    }

    public void setIncomeExpenditurePlanList(List<ReviewIncomeExpenditurePlan> incomeExpenditurePlanList) {
        this.incomeExpenditurePlanList = incomeExpenditurePlanList;
    }

    public boolean isRespRepay() {
        return respRepay;
    }

    public void setRespRepay(boolean respRepay) {
        this.respRepay = respRepay;
    }

    public boolean isMortgageInfo() {
        return mortgageInfo;
    }

    public void setMortgageInfo(boolean mortgageInfo) {
        this.mortgageInfo = mortgageInfo;
    }

    public String getPlanIrrStr() {
        return planIrrStr;
    }

    public void setPlanIrrStr(String planIrrStr) {
        this.planIrrStr = planIrrStr;
    }

    public BigDecimal getPlanIrr() {
        return planIrr;
    }

    public void setPlanIrr(BigDecimal planIrr) {
        this.planIrr = planIrr;
    }

    public List<ReviewRltRepay> getRltRepayList() {
        return rltRepayList;
    }

    public void setRltRepayList(List<ReviewRltRepay> rltRepayList) {
        this.rltRepayList = rltRepayList;
    }

    public String getBusinessNumber() {
        return businessNumber;
    }

    public void setBusinessNumber(String businessNumber) {
        this.businessNumber = businessNumber;
    }

    public BigDecimal getAccountsReceivableNetAmountTotal() {
        return accountsReceivableNetAmountTotal;
    }

    public void setAccountsReceivableNetAmountTotal(BigDecimal accountsReceivableNetAmountTotal) {
        this.accountsReceivableNetAmountTotal = accountsReceivableNetAmountTotal;
    }

    public List<ReviewBasicContract> getReviewBasicContractList() {
        return reviewBasicContractList;
    }

    public void setReviewBasicContractList(List<ReviewBasicContract> reviewBasicContractList) {
        this.reviewBasicContractList = reviewBasicContractList;
    }

    public ReviewTransferReceivable getReviewTransferReceivable() {
        return reviewTransferReceivable;
    }

    public void setReviewTransferReceivable(ReviewTransferReceivable reviewTransferReceivable) {
        this.reviewTransferReceivable = reviewTransferReceivable;
    }

    public List<ReviewMortgageBasicInfo> getMortgageBasicInfoList() {
        return mortgageBasicInfoList;
    }

    public void setMortgageBasicInfoList(List<ReviewMortgageBasicInfo> mortgageBasicInfoList) {
        this.mortgageBasicInfoList = mortgageBasicInfoList;
    }
}
